@@ -2,11 +2,14 @@ __author__ = 'Brian M Anderson'
 
 # Created on 12/31/2020
 import os
+from .Services.DicomBases import ImageBase, RDBase, RTBase, PlanBase, PyDicomKeys, SitkDicomKeys
+from .Services.StaticScripts import poly2mask, add_to_mask
+from .Viewer import plot_scroll_Image
 from tqdm import tqdm
 import typing
 import pydicom
 import numpy as np
-from pydicom.tag import Tag, BaseTag
+from pydicom.tag import Tag
 import SimpleITK as sitk
 from skimage.measure import label, regionprops, find_contours
 from threading import Thread
@@ -14,14 +17,8 @@ from multiprocessing import cpu_count
 from queue import *
 import pandas as pd
 import copy
-import cv2
 from typing import List, Dict
 
-from .Viewer import plot_scroll_Image
-
-PyDicomKeys = Dict[str, BaseTag]  # Example: {"MyNamedRTPlan": Tag((0x300a, 0x002))}
-SitkDicomKeys = Dict[str, str]  # Example: {"MyPatientName": "0010|0010"}
-
 
 def contour_worker(A):
     q, kwargs = A
@@ -127,221 +124,6 @@ class PointOutputMakerClass(object):
                 self.contour_dict[i].append(np.asarray(contour))
 
 
-def poly2mask(vertex_row_coords: np.array, vertex_col_coords: np.array,
-              shape: tuple) -> np.array:
-    """[converts polygon coordinates to filled boolean mask]
-
-    Args:
-        vertex_row_coords (np.array): [row image coordinates]
-        vertex_col_coords (np.array): [column image coordinates]
-        shape (tuple): [image dimensions]
-
-    Returns:
-        [np.array]: [filled boolean polygon mask with vertices at
-                     (row, col) coordinates]
-    """
-    xy_coords = np.array([vertex_col_coords, vertex_row_coords])
-    coords = np.expand_dims(xy_coords.T, 0)
-    mask = np.zeros(shape)
-    cv2.fillPoly(mask, coords, 1)
-    return np.array(mask, dtype=bool)
-
-
-class DICOMBase(object):
-    PatientID: str = None
-    SeriesInstanceUID: str = None
-    SOPInstanceUID: str = None
-    StudyInstanceUID: str = None
-    file: str = None
-    path: typing.Union[str, bytes, os.PathLike] = None
-    additional_tags: Dict
-
-    def load_info(self, *args, **kwargs):
-        pass
-
-
-class RDBase(DICOMBase):
-    SOPInstanceUID: str = None
-    Description: str = None
-    ReferencedStructureSetSOPInstanceUID: str = None
-    ReferencedPlanSOPInstanceUID: str = None
-
-    def __init__(self):
-        self.additional_tags = dict()
-
-    def load_info(self, sitk_dicom_reader, sitk_string_keys: SitkDicomKeys = None):
-        ds = pydicom.read_file(sitk_dicom_reader.GetFileName())
-        self.SeriesInstanceUID = ds.SeriesInstanceUID
-        self.ReferencedStructureSetSOPInstanceUID = ds.ReferencedStructureSetSequence[0].ReferencedSOPInstanceUID \
-            if "ReferencedStructureSetSequence" in ds.values() else None
-        self.ReferencedPlanSOPInstanceUID = ds.ReferencedRTPlanSequence[0].ReferencedSOPInstanceUID
-        self.StudyInstanceUID = sitk_dicom_reader.GetMetaData("0020|000d")
-        if "0008|103e" in sitk_dicom_reader.GetMetaDataKeys():
-            self.Description = sitk_dicom_reader.GetMetaData("0008|103e")
-        self.path = sitk_dicom_reader.GetFileName()
-        self.SOPInstanceUID = sitk_dicom_reader.GetMetaData("0008|0018")
-        if sitk_string_keys is not None:
-            for string in sitk_string_keys:
-                key = sitk_string_keys[string]
-                if key in sitk_dicom_reader.GetMetaDataKeys():
-                    try:
-                        self.additional_tags[string] = sitk_dicom_reader.GetMetaData(key)
-                    except:
-                        continue
-
-
-class PlanBase(DICOMBase):
-    PlanLabel: str
-    ReferencedStructureSetSOPInstanceUID: str
-    ReferencedDoseSOPUID: str
-    Description: str
-
-    def __init__(self):
-        self.additional_tags = {}
-
-    def load_info(self, ds: pydicom.Dataset, path: typing.Union[str, bytes, os.PathLike],
-                  pydicom_string_keys: PyDicomKeys = None):
-        refed_structure_uid = ds.ReferencedStructureSetSequence[0].ReferencedSOPInstanceUID
-        refed_dose_uid = ds.DoseReferenceSequence[0].DoseReferenceUID
-        plan_label = None
-        if Tag((0x300a, 0x002)) in ds.keys():
-            plan_label = ds.RTPlanLabel
-        self.path = path
-        self.SOPInstanceUID = ds.SOPInstanceUID
-        self.PlanLabel = plan_label
-        self.ReferencedStructureSetSOPInstanceUID = refed_structure_uid
-        self.ReferencedDoseSOPUID = refed_dose_uid
-        self.Description = ds.StudyDescription
-        if pydicom_string_keys is not None:
-            for string in pydicom_string_keys:
-                key = pydicom_string_keys[string]
-                if key in ds.keys():
-                    try:
-                        self.additional_tags[string] = ds[key].value
-                    except:
-                        continue
-
-
-class RTBase(DICOMBase):
-    ROI_Names: List[str]
-    ROIs_In_Structure: Dict[str, str]
-    referenced_series_instance_uid: str
-    Plans: Dict[str, PlanBase]
-    Doses: Dict[str, RDBase]
-    CodeAssociations: Dict[str, List[str]]
-
-    def __init__(self):
-        self.Plans = dict()
-        self.Doses = dict()
-        self.additional_tags = dict()
-        self.ROI_Names = []
-        self.ROIs_In_Structure = dict()
-
-    def load_info(self, ds: pydicom.Dataset, path: typing.Union[str, bytes, os.PathLike],
-                  pydicom_string_keys: PyDicomKeys = None):
-        self.StudyInstanceUID = ds.StudyInstanceUID
-        for referenced_frame_of_reference in ds.ReferencedFrameOfReferenceSequence:
-            for referred_study_sequence in referenced_frame_of_reference.RTReferencedStudySequence:
-                for referred_series in referred_study_sequence.RTReferencedSeriesSequence:
-                    refed_series_instance_uid = referred_series.SeriesInstanceUID
-                    if Tag((0x3006, 0x020)) in ds.keys():
-                        ROI_Structure = ds.StructureSetROISequence
-                    else:
-                        ROI_Structure = []
-                    if Tag((0x3006, 0x080)) in ds.keys():
-                        ROI_Observation = ds.RTROIObservationsSequence
-                    else:
-                        ROI_Observation = []
-                    code_strings = {}
-                    for Observation in ROI_Observation:
-                        if Tag((0x3006, 0x086)) in Observation:
-                            code_strings[Observation.ReferencedROINumber] = \
-                                Observation.RTROIIdentificationCodeSequence[0].CodeValue
-                    roi_structure_code_and_names = {}
-                    rois = []
-                    for Structures in ROI_Structure:
-                        roi_name = Structures.ROIName.lower()
-                        rois.append(roi_name)
-                        roi_number = Structures.ROINumber
-                        if roi_number in code_strings:
-                            structure_code = code_strings[roi_number]
-                            if structure_code not in roi_structure_code_and_names:
-                                roi_structure_code_and_names[structure_code] = []
-                            if roi_name not in roi_structure_code_and_names[structure_code]:
-                                roi_structure_code_and_names[structure_code].append(roi_name)
-                        if roi_name not in self.ROIs_In_Structure:
-                            self.ROIs_In_Structure[roi_name] = roi_number
-                    self.path = path
-                    self.ROI_Names = rois
-                    self.SeriesInstanceUID = refed_series_instance_uid
-                    self.SOPInstanceUID = ds.SOPInstanceUID
-                    self.CodeAssociations = roi_structure_code_and_names
-                    if pydicom_string_keys is not None:
-                        for string in pydicom_string_keys:
-                            key = pydicom_string_keys[string]
-                            if key in ds.keys():
-                                try:
-                                    self.additional_tags[string] = ds[key].value
-                                except:
-                                    continue
-
-
-class ImageBase(DICOMBase):
-    Description: str = None
-    slice_thickness: float = None
-    pixel_spacing_x: float = None
-    pixel_spacing_y: float = None
-    SOPs: typing.List[str]
-    files: typing.List[str]
-    RTs: Dict[str, RTBase]
-    RDs: Dict[str, RDBase]
-    RPs: Dict[str, PlanBase]
-
-    def __init__(self):
-        self.RTs = dict()
-        self.RPs = dict()
-        self.RDs = dict()
-        self.additional_tags = dict()
-
-    def load_info(self, dicom_names: typing.List[str], sitk_dicom_reader: sitk.ImageFileReader,
-                  path: typing.Union[str, bytes, os.PathLike],
-                  sitk_string_keys: SitkDicomKeys = None):
-        """
-        Args:
-            dicom_names:
-            sitk_dicom_reader:
-            path:
-            sitk_string_keys:
-
-        Returns:
-
-        """
-        self.SeriesInstanceUID = sitk_dicom_reader.GetMetaData("0020|000e")
-        patientID = sitk_dicom_reader.GetMetaData("0010|0020")
-        while len(patientID) > 0 and patientID[-1] == ' ':
-            patientID = patientID[:-1]
-        self.PatientID = patientID
-        meta_keys = sitk_dicom_reader.GetMetaDataKeys()
-        self.files = dicom_names
-        if "0008|103e" in meta_keys:
-            self.Description = sitk_dicom_reader.GetMetaData("0008|103e")
-        if "0028|0030" in meta_keys:
-            pixel_spacing_x, pixel_spacing_y = sitk_dicom_reader.GetMetaData("0028|0030").strip(' ').split('\\')
-            self.pixel_spacing_x, self.pixel_spacing_y = float(pixel_spacing_x), float(pixel_spacing_y)
-        if "0018|0050" in meta_keys:
-            self.slice_thickness = float(sitk_dicom_reader.GetMetaData("0018|0050"))
-        self.StudyInstanceUID = sitk_dicom_reader.GetMetaData("0020|000d")
-        self.path = path
-        if sitk_string_keys is not None:
-            for string in sitk_string_keys:
-                key = sitk_string_keys[string]
-                if key in sitk_dicom_reader.GetMetaDataKeys():
-                    try:
-                        self.additional_tags[string] = sitk_dicom_reader.GetMetaData(key)
-                    except:
-                        continue
-
-
 def add_images_to_dictionary(images_dictionary: Dict[str, ImageBase], dicom_names: typing.List[str],
                              sitk_dicom_reader: sitk.ImageFileReader, path: typing.Union[str, bytes, os.PathLike],
                              sitk_string_keys: SitkDicomKeys = None):
@@ -430,18 +212,6 @@ def return_template_dictionary():
     return template_dictionary
 
 
-def add_to_mask(mask, z_value, r_value, c_value, mask_value=1):
-    mask[int(np.floor(z_value)), int(np.floor(r_value)), int(np.floor(c_value))] = mask_value
-    mask[int(np.floor(z_value)), int(np.ceil(r_value)), int(np.floor(c_value))] = mask_value
-    mask[int(np.floor(z_value)), int(np.floor(r_value)), int(np.ceil(c_value))] = mask_value
-    mask[int(np.floor(z_value)), int(np.ceil(r_value)), int(np.ceil(c_value))] = mask_value
-    mask[int(np.ceil(z_value)), int(np.floor(r_value)), int(np.floor(c_value))] = mask_value
-    mask[int(np.ceil(z_value)), int(np.ceil(r_value)), int(np.floor(c_value))] = mask_value
-    mask[int(np.ceil(z_value)), int(np.floor(r_value)), int(np.ceil(c_value))] = mask_value
-    mask[int(np.ceil(z_value)), int(np.ceil(r_value)), int(np.ceil(c_value))] = mask_value
-    return None
-
-
 class AddDicomToDictionary(object):
     def __init__(self, plan_pydicom_string_keys: PyDicomKeys = Dict or None,
                  struct_pydicom_string_keys: PyDicomKeys = Dict or None,
@@ -537,42 +307,6 @@ class DicomReaderWriter(object):
         self.rois_in_index_dict = {}
         self.rt_dictionary = {}
         self.mask_dictionary = {}
-        self.plan_pydicom_string_keys = plan_pydicom_string_keys
-        self.struct_pydicom_string_keys = struct_pydicom_string_keys
-        self.image_sitk_string_keys = image_sitk_string_keys
-        self.dose_sitk_string_keys = dose_sitk_string_keys
-        self.images_dictionary = {}
-        self.rd_dictionary = {}
-        self.rp_dictionary = {}
-        if Contour_Names is None:
-            Contour_Names = []
-        if series_instances_dictionary is None:
-            series_instances_dictionary = {}
-        self.series_instances_dictionary = series_instances_dictionary
-        self.get_dose_output = get_dose_output
-        self.require_all_contours = require_all_contours
-        self.flip_axes = flip_axes
-        self.create_new_RT = create_new_RT
-        if associations is None:
-            associations = []
-        self.associations = associations
-        self.Contour_Names = Contour_Names
-        self.set_contour_names_and_associations(contour_names=Contour_Names, associations=associations,
-                                                check_contours=False)
-        self.__set_description__(description)
-        self.__set_iteration__(iteration)
-        self.arg_max = arg_max
-        if template_dir is None or not os.path.exists(template_dir):
-            template_dir = os.path.join(os.path.split(__file__)[0], 'template_RS.dcm')
-        self.template_dir = template_dir
-        self.template = True
-        self.delete_previous_rois = delete_previous_rois
-        self.reader = sitk.ImageSeriesReader()
-        self.image_reader = sitk.ImageFileReader()
-        self.image_reader.LoadPrivateTagsOn()
-        self.reader.MetaDataDictionaryArrayUpdateOn()
-        self.reader.LoadPrivateTagsOn()
-        self.reader.SetOutputPixelType(sitk.sitkFloat32)
         self.dicom_handle_uid = None
         self.dicom_info_uid = None
         self.RS_struct_uid = None
@@ -584,6 +318,44 @@ class DicomReaderWriter(object):
         self.all_rois = []
         self.roi_groups = {}
         self.indexes_with_contours = []
+        self.plan_pydicom_string_keys = plan_pydicom_string_keys
+        self.struct_pydicom_string_keys = struct_pydicom_string_keys
+        self.image_sitk_string_keys = image_sitk_string_keys
+        self.dose_sitk_string_keys = dose_sitk_string_keys
+        self.images_dictionary = {}
+        self.rd_dictionary = {}
+        self.rp_dictionary = {}
+        if series_instances_dictionary is None:
+            series_instances_dictionary = {}
+        self.series_instances_dictionary = series_instances_dictionary
+        self.get_dose_output = get_dose_output
+        self.require_all_contours = require_all_contours
+        self.flip_axes = flip_axes
+        self.create_new_RT = create_new_RT
+        self.arg_max = arg_max
+        if template_dir is None or not os.path.exists(template_dir):
+            template_dir = os.path.join(os.path.split(__file__)[0], 'template_RS.dcm')
+        self.template_dir = template_dir
+        self.template = True
+        self.delete_previous_rois = delete_previous_rois
+        self.associations = associations
+        if Contour_Names is None:
+            self.Contour_Names = []
+        else:
+            self.Contour_Names = Contour_Names
+        self.__initialize_reader__()
+        self.set_contour_names_and_associations(contour_names=Contour_Names, associations=associations,
+                                                check_contours=False)
+        self.__set_description__(description)
+        self.__set_iteration__(iteration)
+
+    def __initialize_reader__(self):
+        self.reader = sitk.ImageSeriesReader()
+        self.image_reader = sitk.ImageFileReader()
+        self.image_reader.LoadPrivateTagsOn()
+        self.reader.MetaDataDictionaryArrayUpdateOn()
+        self.reader.LoadPrivateTagsOn()
+        self.reader.SetOutputPixelType(sitk.sitkFloat32)
 
     def set_index(self, index: int):
         self.index = index
@@ -592,6 +364,35 @@ class DicomReaderWriter(object):
         else:
             self.rois_in_loaded_index = []
 
+    def __mask_empty_mask__(self) -> None:
+        if self.dicom_handle:
+            self.image_size_cols, self.image_size_rows, self.image_size_z = self.dicom_handle.GetSize()
+            self.mask = np.zeros(
+                [self.dicom_handle.GetSize()[-1], self.image_size_rows, self.image_size_cols, len(self.Contour_Names) + 1],
+                dtype=np.int8)
+            self.annotation_handle = sitk.GetImageFromArray(self.mask)
+
+    def __reset_mask__(self):
+        self.__mask_empty_mask__()
+        self.mask_dictionary = {}
+
+    def __reset__(self):
+        self.__reset_RTs__()
+        self.rd_study_instance_uid = None
+        self.dicom_handle_uid = None
+        self.dicom_info_uid = None
+        self.series_instances_dictionary = {}
+        self.rt_dictionary = {}
+        self.images_dictionary = {}
+        self.mask_dictionary = {}
+
+    def __reset_RTs__(self):
+        self.all_rois = []
+        self.roi_groups = {}
+        self.indexes_with_contours = []
+        self.RS_struct_uid = None
+        self.RTs_with_ROI_Names = {}
+
     def __compile__(self):
         """
         The goal of this is to combine image, rt, and dose dictionaries based on the SeriesInstanceUIDs
@@ -767,35 +568,6 @@ class DicomReaderWriter(object):
                 self.series_instances_dictionary[index] = template
         self.__check_if_all_contours_present__()
 
-    def __mask_empty_mask__(self) -> None:
-        if self.dicom_handle:
-            self.image_size_cols, self.image_size_rows, self.image_size_z = self.dicom_handle.GetSize()
-            self.mask = np.zeros(
-                [self.dicom_handle.GetSize()[-1], self.image_size_rows, self.image_size_cols, len(self.Contour_Names) + 1],
-                dtype=np.int8)
-            self.annotation_handle = sitk.GetImageFromArray(self.mask)
-
-    def __reset_mask__(self):
-        self.__mask_empty_mask__()
-        self.mask_dictionary = {}
-
-    def __reset__(self):
-        self.__reset_RTs__()
-        self.rd_study_instance_uid = None
-        self.dicom_handle_uid = None
-        self.dicom_info_uid = None
-        self.series_instances_dictionary = {}
-        self.rt_dictionary = {}
-        self.images_dictionary = {}
-        self.mask_dictionary = {}
-
-    def __reset_RTs__(self):
-        self.all_rois = []
-        self.roi_groups = {}
-        self.indexes_with_contours = []
-        self.RS_struct_uid = None
-        self.RTs_with_ROI_Names = {}
-
     def set_contour_names_and_associations(self, contour_names: List[str] = None,
                                            associations: List[ROIAssociationClass] = None, check_contours=True):
         if contour_names is not None:
@@ -1390,10 +1162,11 @@ class DicomReaderWriter(object):
                 if ROI_Name.lower() in self.Contour_Names:
                     true_name = ROI_Name.lower()
                 else:
-                    for assocation in self.associations:
-                        if ROI_Name.lower() in assocation.other_names:
-                            true_name = assocation.roi_name
-                            break  # Found the name we wanted
+                    if self.associations:
+                        for association in self.associations:
+                            if ROI_Name.lower() in association.other_names:
+                                true_name = association.roi_name
+                                break  # Found the name we wanted
                 if true_name and true_name in self.Contour_Names:
                     mask = self.__return_mask_for_roi__(RT, ROI_Name)
                     self.mask[..., self.Contour_Names.index(true_name) + 1] += mask
