@@ -1,6 +1,7 @@
 __author__ = 'Brian M Anderson'
 # Created on 10/28/2020
-from .ReaderWriter import DicomReaderWriter, plot_scroll_Image, sitk
+from .ReaderWriter import DicomReaderWriter, sitk
+from .Viewer import plot_scroll_Image
 
 
 if __name__ == '__main__':
